@@ -16,9 +16,7 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "evolution-ews-config.h"
 
 #include "e-ews-backend.h"
 
@@ -507,6 +505,7 @@ add_remote_sources (EEwsBackend *backend)
 				E_SERVER_SIDE_SOURCE (source), TRUE);
 			e_server_side_source_set_remote_deletable (
 				E_SERVER_SIDE_SOURCE (source), TRUE);
+			e_source_set_enabled (source, TRUE);
 			e_source_registry_server_add_source (registry, source);
 		} else {
 			GError *error = NULL;
@@ -603,10 +602,6 @@ ews_backend_constructed (GObject *object)
 	e_server_side_source_set_remote_creatable (
 		E_SERVER_SIDE_SOURCE (source), TRUE);
 
-	g_signal_connect (
-		source, "changed",
-		G_CALLBACK (ews_backend_source_changed_cb), object);
-
 	/* Setup the Authentication extension so
 	 * Camel can determine host reachability. */
 	extension_name = E_SOURCE_EXTENSION_AUTHENTICATION;
@@ -671,11 +666,16 @@ ews_backend_populate (ECollectionBackend *backend)
 
 	ews_backend->priv->need_update_folders = TRUE;
 
-	if (!ews_backend->priv->notify_online_id)
+	if (!ews_backend->priv->notify_online_id) {
 		ews_backend->priv->notify_online_id = g_signal_connect (
 			backend, "notify::online",
 			G_CALLBACK (ews_backend_populate), NULL);
 
+		g_signal_connect (
+			source, "changed",
+			G_CALLBACK (ews_backend_source_changed_cb), ews_backend);
+	}
+
 	/* do not do anything, if account is disabled */
 	if (!e_source_get_enabled (source))
 		return;
@@ -1043,6 +1043,11 @@ ews_backend_authenticate_sync (EBackend *backend,
 		e_collection_backend_authenticate_children (E_COLLECTION_BACKEND (backend), credentials);
 
 		e_ews_backend_sync_folders (ews_backend, NULL, ews_backend_folders_synced_cb, NULL);
+	} else if (e_ews_connection_utils_get_without_password (ews_settings) &&
+		   result == E_SOURCE_AUTHENTICATION_REJECTED &&
+		   !e_named_parameters_exists (credentials, E_SOURCE_CREDENTIAL_PASSWORD)) {
+		e_ews_connection_utils_force_off_ntlm_auth_check ();
+		result = E_SOURCE_AUTHENTICATION_REQUIRED;
 	}
 
 	return result;
@@ -1153,7 +1158,7 @@ e_ews_backend_ref_connection_sync (EEwsBackend *backend,
 
 	settings = ews_backend_get_settings (backend);
 	hosturl = camel_ews_settings_dup_hosturl (settings);
-	connection = e_ews_connection_new (hosturl, settings);
+	connection = e_ews_connection_new_full (hosturl, settings, FALSE);
 	g_free (hosturl);
 
 	e_binding_bind_property (
