@@ -16,9 +16,7 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "evolution-ews-config.h"
 
 #include "e-ews-ooo-notificator.h"
 #include "camel/camel-ews-store.h"
