@@ -85,7 +85,6 @@ typedef enum {
 	EWS_SEARCH_AD_CONTACTS,
 	EWS_SEARCH_CONTACTS,
 	EWS_SEARCH_CONTACTS_AD
-
 } EwsContactsSearchScope;
 
 typedef enum {
@@ -386,9 +385,6 @@ void		e_ews_notification_event_free	(EEwsNotificationEvent *event);
 void		ews_oal_free			(EwsOAL *oal);
 void		ews_oal_details_free		(EwsOALDetails *details);
 
-void		e_ews_connection_utils_unref_in_thread
-						(gpointer object);
-
 GType		e_ews_connection_get_type	(void);
 EEwsConnection *e_ews_connection_new		(const gchar *uri,
 						 CamelEwsSettings *settings);
@@ -909,6 +905,15 @@ gboolean	e_ews_connection_move_items_sync
 						 GSList **items_ret,
 						 GCancellable *cancellable,
 						 GError **error);
+gboolean	e_ews_connection_move_items_in_chunks_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *folder_id,
+						 gboolean docopy,
+						 const GSList *ids,
+						 GSList **items,
+						 GCancellable *cancellable,
+						 GError **error);
 
 void		e_ews_connection_create_attachments
 						(EEwsConnection *cnc,
